SystemOrganization addCategory: #'Zinc-WWS-Client'!

Workspace subclass: #ZnWorkspace
	instanceVariableNames: 'url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WWS-Client'!

!ZnWorkspace commentStamp: 'SvenVanCaekenberghe 1/24/2012 23:17' prior: 0!
ZnWorkspace is an experimental subclass of Workspace that adds the ability to access workspaces shared using the http://ws.stfx.eu web service.

For example, try:

	ZnWorkspace openUrl: 'http://ws.stfx.eu/1WS4U'.
!

!ZnWorkspace class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/24/2012 22:22'!
openUrl: url
	"Open a new workspace with the contents of the resource at url."
	
	"self openUrl: 'http://ws.stfx.eu/1WS4U'' "
	
	^ self new
		open;
		openUrl: url;
		yourself! !

!ZnWorkspace methodsFor: 'window menu' stamp: 'PhilippeBack 9/8/2014 08:49'!
addModelItemsToWindowMenu: aMenu 
	super addModelItemsToWindowMenu: aMenu.
	
	aMenu addLine.
	
	aMenu
		add: 'Open web workspace...'
		target: self
		action: #openWWSCommand.
	aMenu lastItem icon: Smalltalk ui icons smallOpenIcon.

	aMenu
		add: 'Save as web workspace...'
		target: self
		action: #saveAsWWSCommand.
	aMenu lastItem icon:  Smalltalk ui icons smallSaveAsIcon.
	
	url ifNotNil: [
		aMenu
			add: 'Reload web workspace'
			target: self
			action: #reloadWWSCommand.
			
		aMenu
			add: 'Copy web workspace URL'
			target: self
			action: #copyWWSUrlCommand ]
! !

!ZnWorkspace methodsFor: 'window menu' stamp: 'SvenVanCaekenberghe 1/25/2012 20:21'!
copyWWSUrlCommand
	Clipboard clipboardText: url asString! !

!ZnWorkspace methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/28/2012 19:51'!
openUrl: urlObject
	| accessor |
	(urlObject isNil or: [ urlObject isEmpty ]) 
		ifTrue: [ ^ self ].
	accessor := ZnWebWorkspaceAccessor new.
	url := urlObject asZnUrl inContextOf: accessor endpoint.
	self contents: (accessor get: url).
	self label: url asString.
	self changed: #contents! !

!ZnWorkspace methodsFor: 'window menu' stamp: 'SvenVanCaekenberghe 1/28/2012 13:04'!
openWWSCommand
	| userWarned |
	userWarned := true.
	(url isNil and: [ self contents isEmpty not or: [self hasUnacceptedEdits ]])
		ifTrue: [ userWarned := self warnUser ].
	(url isNil not and: [self hasUnacceptedEdits])
		ifTrue: [ userWarned := self warnUser ].
	userWarned 
		ifTrue: [ | input |
			input := UIManager default request: 'URL or key' initialAnswer: '' title: 'http://ws.stfx.eu'. 
			self openUrl: input ]! !

!ZnWorkspace methodsFor: 'window menu' stamp: 'SvenVanCaekenberghe 1/28/2012 19:52'!
reloadWWSCommand
	self warnUser
		ifTrue: [ self openUrl: url ]! !

!ZnWorkspace methodsFor: 'window menu' stamp: 'SvenVanCaekenberghe 1/30/2012 22:20'!
saveAsWWSCommand
	(UIManager default confirm: 'Are you sure you want to publish the contents of this workspace ?')
		ifFalse: [ ^ self ].
	self codeTextMorph 
		ifNotNil: [ :morph | | stringToSave |
			stringToSave :=  morph text string.
			url := ZnWebWorkspaceAccessor new post: stringToSave.
			self label: url asString.
			self contents: stringToSave.
			self changed: #contents ]! !

Object subclass: #ZnWebWorkspaceAccessor
	instanceVariableNames: 'endpoint'
	classVariableNames: 'DefaultEndpoint'
	poolDictionaries: ''
	category: 'Zinc-WWS-Client'!

!ZnWebWorkspaceAccessor commentStamp: 'SvenVanCaekenberghe 1/29/2012 19:25' prior: 0!
ZnWebWorkspaceAccessor accesses workspace contents shared using the http://ws.stfx.eu web service.

Example usage:

	ZnWebWorkspaceAccessor new get: 'http://ws.stfx.eu/1WS4U'.

or

	ZnWebWorkspaceAccessor new get: '1WS4U'.

This global setting helps when testing:

	ZnWebWorkspaceAccessor defaultEndpoint: 'http://localhost:1701'.!

!ZnWebWorkspaceAccessor class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/29/2012 19:25'!
defaultEndpoint
	^ DefaultEndpoint ifNil: [ DefaultEndpoint := 'http://ws.stfx.eu' asZnUrl ]! !

!ZnWebWorkspaceAccessor class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 11:05'!
defaultEndpoint: aUrlObject
	DefaultEndpoint := aUrlObject ifNotNil: [ aUrlObject asZnUrl ]! !

!ZnWebWorkspaceAccessor methodsFor: 'public' stamp: 'SvenVanCaekenberghe 1/29/2012 19:23'!
endpoint
	^ endpoint ifNil: [ endpoint := self class defaultEndpoint ]! !

!ZnWebWorkspaceAccessor methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/28/2012 18:48'!
endpoint: anUrlObject
	endpoint := anUrlObject asZnUrl ! !

!ZnWebWorkspaceAccessor methodsFor: 'public' stamp: 'SvenVanCaekenberghe 1/30/2012 22:11'!
get: url
	"Access the shared web workspace at url 
	and return its contents as a String.
	Signals an error when something goes wrong."
	
	| contents |
	contents := self httpClient get: url.
	^ String new: contents size streamContents: [ :stream |
		contents linesDo: [ :each | stream nextPutAll: each; cr ] ]! !

!ZnWebWorkspaceAccessor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/24/2012 21:36'!
httpClient
	^ ZnClient new
		systemPolicy;
		beOneShot;
		accept: ZnMimeType textPlain;
		url: self endpoint;
		yourself! !

!ZnWebWorkspaceAccessor methodsFor: 'public' stamp: 'SvenVanCaekenberghe 1/24/2012 23:45'!
post: string
	"Publish the string as a new shared web workspace
	and return the newly created url.
	Signals an error when something goes wrong."
	
	| entity client url |
	entity := ZnEntity text: string asString.
	(client := self httpClient)
		entity: entity; 
		post.
	url := client response location.
	^ url asZnUrl inContextOf: self endpoint ! !
